import {Component, OnDestroy, OnInit} from '@angular/core';
import {CardInfo} from '../interface/cardInfo';
import {TabuMiddlewareService} from '../dao/TabuMiddlewareService';
import {ActivatedRoute, Router} from '@angular/router';
import {GameStatus} from '../interface/gameStatus';
import {IsAllowedToPlay} from '../dao/isAllowedToPlay';
import {SocketDataService} from '../dao/socketDataService';
import {Subject} from 'rxjs';
import {takeUntil} from 'rxjs/operators';

@Component({
  selector: 'app-game',
  templateUrl: './game.component.html',
  styleUrls: ['./game.component.scss']
})
export class GameComponent implements OnInit, OnDestroy {

  private ngUnsubscribe = new Subject();
  subNewCard: any;
  subEndRound: any;
  sessionName = '';
  isWatchdog = false;
  isExplainer = false;
  TIME_LIMIT = 100;
  timePassed = 0;
  timeRemaining = 60;
  timerInterval = 0;
  timeLeft = this.TIME_LIMIT;
  cardInfo: CardInfo = {
    cardID: -1,
    solution: '',
    tabu1: '',
    tabu2: '',
    tabu3: '',
    tabu4: '',
    tabu5: ''
  };

  gameStatus: GameStatus = {
    sessionID: -1,
    red: 0,
    blue: 0,
    redTurn: true,
    activeExplainer: 1,
    activeCard: 1
  };
  team = '';

  color = 'primary';
  value = 50;

  constructor(private service: TabuMiddlewareService,
              private router: Router,
              private activatedRoute: ActivatedRoute,
              private isAllowedToPlay: IsAllowedToPlay,
              private socketDataService: SocketDataService) {
  }

  ngOnInit(): void {
    this.activatedRoute.paramMap.subscribe(params => {
      this.sessionName = String(params.get('sessionName')).toLowerCase();
      this.team = String(params.get('team'));
      this.getSocketData();
      const url = this.router.url.split('/');
      const checkUser = url[url.length - 1];
      if (checkUser === 'watchdog') {
        if (!this.isAllowedToPlay.isAllowed || this.isAllowedToPlay.role !== 'watchdog') {
          this.router.navigate([this.sessionName + '/' + this.team]);
        } else {
          this.isWatchdog = true;
        }
      } else if (checkUser === 'explainer') {
        if (!this.isAllowedToPlay.isAllowed || this.isAllowedToPlay.role !== 'explainer') {
          this.router.navigate([this.sessionName + '/' + this.team]);
        } else {
          this.isExplainer = true;
          //this.service.getS2C({spielname: this.sessionName});
        }
      }
      console.log('FILL GAME');
      this.fillGamestatus();
    });
    this.startTimer();
  }

  ngOnDestroy(): void {
    this.unsubscribeAll();
  }

  getSocketData(): void {
    this.subNewCard = this.socketDataService.getNewCard(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(data => {
        console.log('GAME SOCKET CARD: ', data);
        this.service.getCard({cardID: data}).then(value => {
          this.fillNewCard(value);
          this.fillGamestatus();
        });
      });
    this.subEndRound = this.socketDataService.getNewRound(this.sessionName)
      .pipe(takeUntil(this.ngUnsubscribe))
      .subscribe(data => {
        console.log('GAME SOCKET END ROUND: ', data);
        if (!JSON.parse(data)) {
          console.log('GAME END: ', data);
          this.unsubscribeAll();
          this.router.navigate([this.sessionName + '/' + this.team]);
        }
      });
  }

  onTimesUp(): void {
    this.unsubscribeAll();
    this.router.navigate([this.sessionName + '/' + this.team]);
  }

  startTimer(): void {
    this.startCountdown();
    this.timerInterval = setInterval(() => {
      this.timePassed = this.timePassed += 1;
      this.timeLeft = (this.TIME_LIMIT - this.timePassed);
      if (this.timeLeft === 0) {
        console.log('Times up!');
        //this.onTimesUp();
      }
    }, 593);
  }

  startCountdown(): void {
    setInterval(() => {
      this.timeRemaining = this.timeRemaining -= 1;
    }, 1000);
  }

  rightAnswer(): void {
    this.service.addPoint
    ({
      spielname: this.sessionName, team: this.team, red: this.gameStatus.red, blue: this.gameStatus.blue, cardID: this.gameStatus.activeCard
    }).then(value => {
      this.newCard();
      this.gameStatus.red = JSON.parse(value.red);
      this.gameStatus.blue = JSON.parse(value.blue);
    });
  }

  wrongAnswer(): void {
    this.newCard();
    if (this.isExplainer) {
      this.service.removePoint
      ({
        spielname: this.sessionName,
        team: this.team,
        red: this.gameStatus.red,
        blue: this.gameStatus.blue,
        cardID: this.gameStatus.activeCard
      }).then(value => {
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
      });
    } else if (this.isWatchdog) {
      let opennentTeam = '';
      if (this.team === 'red') {
        opennentTeam = 'blue';
      } else if (this.team === 'blue') {
        opennentTeam = 'red';
      }
      this.service.removePoint
      ({
        spielname: this.sessionName,
        team: opennentTeam,
        red: this.gameStatus.red,
        blue: this.gameStatus.blue,
        cardID: this.gameStatus.activeCard
      }).then(value => {
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
      });
    }
  }

  newCard(): void {
    this.service.getS2C({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        this.fillNewCard(value);
      }
    }).catch(reason => console.log(reason));
  }

  fillNewCard(value: any): void {
    this.cardInfo.cardID = JSON.parse(value.cardID);
    this.cardInfo.solution = value.solution;
    this.cardInfo.tabu1 = value.tabu1;
    this.cardInfo.tabu2 = value.tabu2;
    this.cardInfo.tabu3 = value.tabu3;
    this.cardInfo.tabu4 = value.tabu4;
    this.cardInfo.tabu5 = value.tabu5;
  }

  fillGamestatus(): void {
    this.service.getGamestatus({spielname: this.sessionName}).then(value => {
      const status = JSON.parse(value.status);
      if (status) {
        console.log(value);
        this.gameStatus.sessionID = JSON.parse(value.sessionID);
        this.gameStatus.red = JSON.parse(value.red);
        this.gameStatus.blue = JSON.parse(value.blue);
        this.gameStatus.redTurn = JSON.parse(value.redTurn);
        this.gameStatus.activeExplainer = JSON.parse(value.activeExplainer);
        this.gameStatus.activeCard = JSON.parse(value.activeCard);
        console.log('SOLUTION: ', this.cardInfo.cardID);
        if (this.cardInfo.cardID === -1) {
          console.log('I am in');
          this.service.getCard({cardID: value.activeCard}).then(value1 => {
            console.log('FIRST CARDS: ', value1);
            this.fillNewCard(value1);
          });
        }
      }
    });
  }

  unsubscribeAll(): void {
    this.subNewCard.unsubscribe();
    this.subEndRound.unsubscribe();

    this.ngUnsubscribe.next();
    this.ngUnsubscribe.complete();
  }
}
